package com.nisum.springboot.controller;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Sort;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.aggregation.Aggregation;
import org.springframework.data.mongodb.core.aggregation.AggregationResults;
import org.springframework.data.mongodb.core.aggregation.GroupOperation;
import org.springframework.data.mongodb.core.aggregation.MatchOperation;
import org.springframework.data.mongodb.core.aggregation.SortOperation;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.nisum.springboot.dto.AgeCount;
import com.nisum.springboot.entity.Employee;
import com.nisum.springboot.entity.Product;
import com.nisum.springboot.service.EmployeeService;

@RestController
@RequestMapping("/employees")
public class EmployeeController {
	
	@Autowired
	private EmployeeService employeeService;
	
	@PostMapping
	List<Employee> saveEmployees(@RequestBody List<Employee> employees){
		return employeeService.saveEmployees(employees);
	}
	
	@GetMapping
	List<Employee> getAllEmployees(){
		return employeeService.getAllEmployees();
	}
	
	@GetMapping("/name/{name}")
	List<Employee> getEmployeeName(@PathVariable String name){
		return employeeService.findByName(name);
	}
	
	@GetMapping("/name/{name}/dept/{department}")
	List<Employee> findByNameAndDept(@PathVariable String name , @PathVariable String department){
		return employeeService.findByNameAndDept(name,department);
	}
	
	@GetMapping("/minage/{minage}/maxage/{maxage}")
	List<Employee> getEmployeesAgeBetween(@PathVariable int minage, @PathVariable int maxage){
		return employeeService.getEmployeesAgeBetween(minage, maxage);
	}
	
	
	@Autowired
	private MongoTemplate mongoTemplate;
	
	@GetMapping("/namewithquery/{name}")
	List<Employee> getEmployeeNameWithQuery(@PathVariable String name){
		Query query= new Query();
		//query.addCriteria(Criteria.where("name").is(name));
		query.addCriteria(Criteria.where("name").regex(name));
		return mongoTemplate.find(query, Employee.class);
	}
	
	@GetMapping("/namewithqueryreg/{name}")
	List<Employee> getEmployeeNamewithReg(@PathVariable String name){
		Query query= new Query();
		query.addCriteria(Criteria.where("name").regex(name));
		return mongoTemplate.find(query, Employee.class);
	}
	
	@GetMapping("/aggregateexample/{age}")
	List<Employee> findByAgeMatch(@PathVariable int age){
		
		MatchOperation  matchoperation=Aggregation.match(new Criteria("age").is(age));
		SortOperation  sortoperation=Aggregation.sort(Sort.by(Sort.Direction.DESC, "age"));
		Aggregation aggregation=Aggregation.newAggregation(matchoperation, sortoperation);
		
		AggregationResults<Employee>  results=mongoTemplate.aggregate(aggregation, "employees", Employee.class);
		//return mongoTemplate.find("", Employee.class);
		
		return results.getMappedResults();
	}
	
	@GetMapping("/aggregategroupexample/{age}")
	List<AgeCount> findByAgeGroup(@PathVariable int age){
		
		GroupOperation  groupByDept=Aggregation.group("age").count().as("count");
		MatchOperation  matchoperation=Aggregation.match(new Criteria("count").is(age));
		SortOperation  sortoperation=Aggregation.sort(Sort.by(Sort.Direction.DESC, "count"));
		Aggregation aggregation=Aggregation.newAggregation(groupByDept,matchoperation, sortoperation);
		
		AggregationResults<AgeCount>  results=mongoTemplate.aggregate(aggregation, "employees", AgeCount.class);
		//return mongoTemplate.find("", Employee.class);
		
		return results.getMappedResults();
	}

}
